// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "bayonnedb.h"
#include <ctype.h>

CDRDriver::CDRDriver() :
Driver(), Keys(137), Keydata("/bayonnedb/cdr")
{
	static KEYDEF keydefs[] = {
	{"expires", "60"},
	{"purge", "5"},
	{NULL, NULL}};

	Load(keydefs);
}

void CDRDriver::Schedule(void)
{
	static int purge = 0;

	if((++purge) >= getPurge())
	{
		purge = 0;
		expire();
	}
}

int CDRDriver::Accept(unsigned char *msgbuf, unsigned msglen)
{
	char *cp;
	char *tok;
	char *pos;
	char *fields[18];
	int i, len;

	typedef struct parse
	{
		char *id;
		int offset;
	} parse_t;

	static parse_t parse[] = {
	{"type=", 1},
	{"country=", 2},
	{"node=", 3},
	{"port=", 4},
	{"policy=", 5},
	{"group=", 5},
	{"calledid=", 6},
	{"dialed=", 6},
	{"dial=", 6},
	{"dnis=", 6},
	{"dnid=", 6},
	{"did=", 6},
	{"caller=", 7},
	{"callerid=", 7},
	{"call=", 7},
	{"ani=", 7},
	{"cid=", 7},
	{"clid=", 7},
	{"dest=", 8},
	{"target=", 8},
	{"account=", 9},
	{"date=", 10},
	{"time=", 11},
	{"duration=", 12},
	{"location=", 13},
	{"cost=", 14},
	{"sessionid=", 15},
	{"sid=", 15},
	{"infodigits=", 16},
	{"ii=", 16},
	{NULL, 0}};

	parse_t *entry;
	
	if(msgbuf[0] != 'C')
		return 0;

	cp = strchr((char *)(msgbuf + 1), ':');
	if(!cp)
		return 0;	

	*cp = 0;

	EnterMutex();
	if(!find((char *)msgbuf + 1))
	{
		add((char *)msgbuf + 1, getExpires());
		msgbuf[msglen] = 0;
		for(i = 0; i < 18; ++i)
			fields[i] = "";
		fields[0] = (char *)msgbuf + 1;
		fields[1] = "I";
		fields[2] = "na";
		fields[12] = "0";
		fields[14] = "0.0";
		tok = strtok_r(cp + 1, " \t\r\n", &pos);
		while(tok)
		{
			entry = parse;
			while(entry->id)
			{
				len = strlen(entry->id);
				if(!strnicmp(tok, entry->id, len))
				{
					fields[entry->offset] = tok + len;	
					break;
				}
				++entry;
			}
			tok = strtok_r(NULL, " \t\r\n", &pos);
		}
		Commit(fields);
	}
	LeaveMutex();
	msgbuf[0] = 'A';
	*cp = ':';
	return (int)(cp - (char *)msgbuf) + 1;
}




